<?php

namespace Drupal\cancel_account\Form;

use Drupal\Core\Config\ConfigFactory;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Password\PasswordInterface;
use Drupal\user\Entity\User;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Form controller for the cancel user account form.
 */
class CancelAccountForm extends FormBase {

  /**
   * The Password Hasher.
   *
   * @var \Drupal\Core\Password\PasswordInterface
   */
  protected $passwordHasher;

  /**
   * The configuration object factory.
   *
   * @var \Drupal\Core\Config\ConfigFactory
   */
  protected $configFactory;

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'cancel_account_form';
  }

  /**
   * Constructs a CancelAccountForm object.
   *
   * @param \Drupal\Core\Password\PasswordInterface $password_hasher
   *   The password hasher.
   * @param \Drupal\Core\Config\ConfigFactory $config_factory
   *   The configuration object factory.
   */
  public function __construct(PasswordInterface $password_hasher, ConfigFactory $config_factory) {
    $this->passwordHasher = $password_hasher;
    $this->configFactory = $config_factory;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('password'),
      $container->get('config.factory')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state, $extra = NULL) {
    // Get user.
    if (!$extra) {
      $account = $this->currentUser();
    }
    else {
      $account = User::load((int) $extra);
    }

    // Get current user.
    $user = $this->currentUser();

    // Build cancel account form.
    if (!$user->isAnonymous() && $user->id() == $account->id()) {
      // Check cancel account permissions and prevent adding form to superadmin
      if ($user->hasPermission('cancel account') && $account->id() != 1) {
        $form['cancel'] = [
          '#type' => 'container',
          '#weight' => -10,
          '#prefix' => '<h2>' . $this->t('Cancel your account') . '</h2>',

        ];
        $form['cancel']['markup'] = [
          '#markup' => $this->t('This action cannot be undone.'),
          '#prefix' => '<p>',
          '#suffix' => '</p>',
        ];
        $form['cancel']['confirm'] = [
          '#type' => 'checkbox',
          '#default_value' => 0,
          '#title' => $this->t('Confirm deleting my account.'),
        ];
        $form['cancel']['pass'] = [
          '#type' => 'password',
          '#title' => $this->t('Current password'),
          '#attributes' => ['autocomplete' => 'off'],
          '#size' => 25,
          '#required' => TRUE,
        ];
        $form['actions'] = ['#type' => 'actions'];
        $form['actions']['submit'] = [
          '#type' => 'submit',
          '#value' => $this->t('Cancel account'),
          '#states' => [
            'enabled' => [
              [
                ':input[name="confirm"]' => ['checked' => TRUE],
                ':input[name="pass"]' => ['filled' => TRUE],
              ],
            ],
          ],
        ];
      }
    }
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    $pass_input = trim($form_state->getValue('pass'));
    if ($pass_input) {
      $user = User::load($this->currentUser()->id());

      // Check user password.
      if (!$this->passwordHasher->check($pass_input, $user->getPassword())) {
        $form_state->setErrorByName('pass', $this->t('The password you provided is incorrect.'));
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $uid = $this->currentUser()->id();
    $config = $this->configFactory->get('user.settings');

    // Get account cancel settings.
    $method = $config->get('cancel_method');
    $edit['user_cancel_notify'] = $config->get('notify.status_canceled');

    // Remove user account finally.
    user_cancel($edit, $uid, $method);
  }

}
